


#include <mioP.h>

extern int CurrentDac;
extern int WgenStat;

volatile int DaqMode = 0;

void mio_SetDaqMode(int mode){
DBGin("mio_SetDaqMode");
DBGprint(DBG_DATA,("Set DAQ-Mode (0x%x)",mode));
DaqMode = mode;
DBGout();
}

/**********************************************************************/
int mio_GetAdc(AdcVal *val){

int stat;
AdcVal buf;
int adcval=0;
int dac;
DBGin("mio_GetAdc");

if(stat = verify_area(VERIFY_WRITE,val,sizeof(AdcVal)))
  return stat;

outw(0x00,MIO_ADCLR);

  /* set ADC UNIPOLAR/BIPOLAR mode */
  if( DaqMode & DAQ_BIPOLAR ) 
    mio_reset_bitcmd(BIPOLAR ,0 );
  else
    mio_set_bitcmd(BIPOLAR ,0 );

  /* set ADC DIFF/RSE mode */
  if( DaqMode & DAQ_DIFF )
    mio_set_bitcmd(DIFF, 0);
  else
    mio_reset_bitcmd(DIFF, 0);

outw(0x00,MIO_SCONV);
while(!(inw(MIO_SR) & FIFOEF )) /* poll status register */
  dac=CurrentDac;


adcval = inw(MIO_ADFIFO); /* read value */

buf.adc = adcval;
buf.dac0 = dac;


DBGprint(DBG_DATA,("( adc=0x%x,dac=%d )",adcval,buf.dac0 ));


memcpy_tofs(val,&buf,sizeof(AdcVal));

DBGout();return OK;
}


/***********************************************************************
 *
 *  Doublebuffer FIFO mechanism
 *
 */

#ifndef MAX_DBUFSIZE
#define MAX_DBUFSIZE 1024
#endif

struct wait_queue *DaqWait = NULL;
struct   semaphore  DaqSem = MUTEX;


unsigned int DblBuf[2][MAX_DBUFSIZE+1];

unsigned int DblBufState = 0;

unsigned int ActualReadCount = 0;
unsigned int ActualWriteCount = 0;
unsigned int ActualResidue    = 0;

unsigned int ActualWrite = 0;
unsigned int ActualRead  = 0;

/***********************************************************************/
void mio_DaqInit(void){
DBGin("mio_DaqInit");


  DblBufState = 0;
  ActualResidue = 0;
  ActualWriteCount = 0;
  ActualReadCount = 0;

  ActualRead = 0;
  ActualWrite = 0;
  DaqMode = 0;

  DBGprint(DBG_DATA,(" %d bytes FIFO-Buffer",2*MAX_DBUFSIZE));

  /*clear A/D circuit */
  outw(0x0000,MIO_ADCLR);

DBGout();
}


/**********************************************************************/
void mio_SwapWrite(void){
  DBGin("mio_SwapWrite");
  ActualWrite = ( ActualWrite ? 0 : 1 );
  DBGprint(DBG_DATA,("Swapped Write to %d",ActualWrite));
  ActualWriteCount = 0;
  if( ActualResidue >= (2*MAX_DBUFSIZE)){ /* overflow buffer bit set */
    DBGprint(DBG_DATA,("Buffer Overflow"));
    DblBufState |= (DB_ERROR | DB_OVERFLOW);
  }
  DBGout();
}
/**********************************************************************/
void mio_SwapRead(void){
DBGin("mio_SwapRead");
      ActualRead = ( ActualRead ? 0 : 1 );
      DBGprint(DBG_DATA,("Swapped Read to %d",ActualRead));
      ActualReadCount = 0;
DBGout();     
}
/**********************************************************************/
void mio_AddDBuf(unsigned int val){
  DBGin("mio_AddDBuf");
  if( ActualWriteCount < MAX_DBUFSIZE ){
    DblBuf[ActualWrite][ActualWriteCount] = val;
    ActualWriteCount ++;
    ActualResidue ++;
  } else {
    mio_SwapWrite();
  }
  if( ActualResidue >= MAX_DBUFSIZE ){
    DblBufState |= DB_HALF;
    if( DaqMode & DAQ_SYNC ) wake_up_interruptible(&DaqWait);
  }
  DBGout();     
}
/**********************************************************************/
unsigned int mio_GetDBuf(void){
  unsigned int ret=0;
  DBGin("mio_GetDBuf");

  if( ActualResidue > 0 ){
    if( ActualReadCount < MAX_DBUFSIZE ){
      ret = DblBuf[ActualRead][ActualReadCount];
      ActualReadCount ++;
      ActualResidue --;
    } else {
      mio_SwapRead();
    }
    if( ActualResidue < MAX_DBUFSIZE )
      DblBufState &= ~DB_HALF;
  } else {
    ret=0;
    DblBufState |= DB_EMPTY;
  }

  DBGprint(DBG_DATA,("residue=%d",ActualResidue));
  return ret;
  DBGout();     
}

/**********************************************************************/
int mio_SyncRead(int *buf, int count ){
int retval,i;
int dummy;
char *tbuf;
DBGin("mio_SyncRead");



if(retval = verify_area(VERIFY_WRITE, buf, count))
  return retval;


if( DblBufState & DB_EMPTY ){
  DBGprint(DBG_DATA,("Buffer Empty"));
  return -EINTR;
}

if( !(DblBufState & DB_HALF) && ( DaqMode & DAQ_RUNNING ) ){
  DBGprint(DBG_DATA,("\nWaiting for DB_HALF"));
  DaqMode |= DAQ_SYNC;
  interruptible_sleep_on(&DaqWait);
  DaqMode &= ~DAQ_SYNC; 
  DBGprint(DBG_DATA,("\nGot DB_HALF"));
} 

  if( DblBufState & DB_OVERFLOW ){
    DBGprint(DBG_DATA,("Double Buffer Overflow"));
    return -EOVERFLOW;
  }

  /* copy block to FS */
for(i=0;i<count && i<ActualResidue ;i++){
  dummy = mio_GetDBuf();
  DBGprint(DBG_DATA,("buf=%d",dummy));
  if( DblBufState & DB_EMPTY ) {
    DBGprint(DBG_DATA,("count=%d",i));
    return i;
  }
  put_fs_long( dummy ,(int *) &(buf[i]));
  /*memcpy_tofs((int *) buf, &dummy ,sizeof(int));*/
}

return count;

DBGout();
}

/**********************************************************************/
void mio_DaqFinish(void){
DBGin("mio_DaqFinish");

  DaqMode &= ~DAQ_RUNNING;
  if ( DaqMode & DAQ_SYNC ) wake_up_interruptible(&DaqWait);
DBGout();
}

/**********************************************************************/
void mio_DaqStart(int mode){



DBGin("mio_DaqStart");
DBGprint(DBG_DATA,("mode (0x%x)",mode));

  mio_DaqInit();

  DaqMode |= (DAQ_RUNNING | mode );

  /*clear A/D circuit */
  outw(0x0000,MIO_ADCLR);

  /* set ADC UNIPOLAR/BIPOLAR mode */
  if( DaqMode & DAQ_BIPOLAR ) 
    mio_reset_bitcmd(BIPOLAR ,0 );
  else
    mio_set_bitcmd(BIPOLAR ,0 );

  /* set ADC DIFF/RSE mode */
  if( DaqMode & DAQ_DIFF )
    mio_set_bitcmd(DIFF, 0);
  else
    mio_reset_bitcmd(DIFF, 0);

  if( DaqMode & DAQ_SCONV ){
    DBGprint(DBG_DATA,("trigger conversion"));
    outw(0x00,MIO_SCONV);          /* trigger first value */
    while(!(inw(MIO_SR) & FIFOEF )); /* sync FIFOEF */
  }

  if( DaqMode & DAQ_DACUP ) printk("WGEN MODE\n");

DBGout();
}

/***********************************************************************/
void mio_DaqIRQ(int status){
unsigned int val;
#if DEBUG
if(dbgMask) printk("DAQ-INTERRUPT (%d)\n",status);
#endif
if( DaqMode & DAQ_RUNNING ){

  while( inw(MIO_SR) & FIFOEF ){  /* read FIFO content into Buffer */
    val = inw(MIO_ADFIFO) | (CurrentDac<<16);
    mio_AddDBuf( val );
#if DEBUG
    if( dbgMask ) printk("val=%d Res=%d",val,ActualResidue);
#endif
  }

    /* trigger next conversion */
    if (DaqMode & DAQ_SCONV){ 
      outw(0x0000,MIO_SCONV);
    }

  if((DaqMode & DAQ_DACUP) && !(WgenStat & WGEN_RUNNING)){
#if DEBUG
    if( dbgMask ) printk("Finish DACUP DAQ Cycle\n");
#endif
    DaqMode &= ~DAQ_RUNNING;
    while ( !(inw(MIO_SR) & FIFOEF)); /* poll status */
    val = inw(MIO_ADFIFO) | (CurrentDac<<16);
    mio_AddDBuf( val );

    if ( DaqMode & DAQ_SYNC ) wake_up_interruptible(&DaqWait);
  }

} else 
  printk("DAQ-INTERRUPT unknown status (0x%x)",status);

}

/***********************************************************************/










