
#include <mioP.h>

#define LINUX_STD_PARAM  struct inode* inode,struct file* filep

static int mio_opened=0;

/***********************************************************************/

void mio_interrupt (int irq){
int status;

     /*printk("MIO-INTERRUPT\n");*/

     status = inw( MIO_SR );
     if( status & ( DACUP | DACUPERR )){
       WavGen_IRQ(status);
     }

     if( status & FIFOEF ){
       mio_DaqIRQ(status);
     }

}

/***********************************************************************/

int  mio_ioctl  (LINUX_STD_PARAM, unsigned int cmd, unsigned long arg){
  DBGin("mio_ioctl");
  
  switch (cmd) {

  case MIOSETDAC0:
    mio_a0out((int) arg );
    break;
  case MIOSETDAC1:
    mio_a1out((int) arg );
    break;


  case MIOADDTRACE:
    WavGen_AddTrace((WaveTrace *) arg);
    break;

  case MIOWGENRESET:
    WavGen_Reset();
    break;

  case MIOWGENSTART:
    WavGen_Start();
    break;

  case MIOWGENCONT:
    WavGen_Continue();
    break;

  case MIOWGENSTOP:
    WavGen_Stop();
    break;

  case MIOWGENSFREQ :
    WavGen_SetFreq((int) arg);
    break;
  case MIOWGENSTIM  :
    WavGen_SetTimebase((int) arg);
    break;

  case MIOWGENSDAC  :
    WavGen_SetDac((int) arg);
    break;

  case MIOSETMUX    :
    mio_SetMux((int) arg);
    break;

  case MIOGETADC    :
    mio_GetAdc((AdcVal *) arg);
    break;

  case MIOWISRUN :
    return Wgen_Running();
    break;

  case MIOWGENSCHA    :
    WavGen_SetChan( (int) arg);
    break;

  case MIOSETMODE :
    mio_SetDaqMode((int) arg);
    break;


  case MIODIOCNF :
    mio_DioConfigure((int) arg);
    break;

  case MIODIORD :
    return mio_Din();
    break;

  case MIODIOWRT :
    mio_Dout((int) arg);
    break;


  case MIORTSIRESET:
    mio_ResetRTSI();
    break;

  case MIORTSISET:
    /* MSB = Pin */
    /* LSB = Mode */

    mio_SelectRTSI((char) (arg>>8),(char) (arg));
    break;

  case MIODEBUG :
#if DEBUG
    dbgMask =((int) arg);
#endif
    break;
  default:
    DBGout();
    return( -EINVAL );
    break;
  }

  DBGout();
}
/***********************************************************************/
int mio_read (LINUX_STD_PARAM, char *buf, int count ){
int retval=0;
  DBGin("mio_read");
    retval = mio_SyncRead((int *) buf,count);
  DBGout();
  return retval;
}
/***********************************************************************/
int  mio_open   (LINUX_STD_PARAM){
  DBGin("mio_open");

  if( mio_opened ) {
    return(-EBUSY);
  }

  if( request_irq(mio_irq, mio_interrupt, SA_INTERRUPT, "atMIO16f" )){
    printk("atMIO: Can't Request IRQ(%d) \n",mio_irq);
    return -EIO;
  }
 
  mio_DaqInit();


  MOD_INC_USE_COUNT;
  mio_opened++;




  DBGout(); 
  return(OK);
}
/***********************************************************************/
void mio_close  (LINUX_STD_PARAM){
  DBGin("mio_close");

  mio_opened--;
  MOD_DEC_USE_COUNT;

  WavGen_Reset();
  mio_DaqInit();

  free_irq(mio_irq);

  DBGout();
}

