
#include <mioP.h>



#include "kernel_version.h"

/* default debugging level */

#if DEBUG
#ifndef DEFAULT_DEBUG
unsigned int   dbgMask  = (DBG_ENTRY | DBG_EXIT | DBG_BRANCH | DBG_DATA | DBG_INTR | DBG_1PPL) & ~DBG_ALL;
#else
unsigned int   dbgMask  = 0;
#endif
#endif


/***************************************************************************************

   Init module functions


****************************************************************************************/

extern int  mio_ioctl  (struct inode *, struct file *, unsigned int, unsigned long);
extern int  mio_open   (struct inode *, struct file *);
extern void mio_close  (struct inode *, struct file *);
extern int  mio_read   (struct inode *, struct file *,char *,int);


struct file_operations mio_fops = {
  NULL,                 /* seek */
  mio_read,             /* read */
  NULL,                 /* write */
  NULL,                 /* readdir */
  NULL,                 /* select */
  mio_ioctl,            /* ioctl */
  NULL,                 /* mmap */
  mio_open,             /* open */
  mio_close             /* release */
};


#ifdef __cplusplus
extern "C" {
#endif

int mio_major = MIO_MAJOR;   /* major number for dynamic configuration */
int mio_base  = MIO_DEFAULT_BASE   ;    /* base address */
int mio_irq   = MIO_DEFAULT_IRQ    ;


/***********************************************************************/

int init_module(void)
{

        char signature=0;

	printk("atMIO16f Driver, (c) 1995 by C.Schroeter\n");
        printk(" -- major(%d), base(0x%x), irq(%d) \n",mio_major, mio_base, mio_irq);

  	DBGin("init_module");

#if DEBUG
	if(dbgMask != 0 )
         	dbgMask |= DBG_1PPL;
#endif
	
	if( register_chrdev(mio_major,"atMIO16f",&mio_fops)){
	  printk("can't get Major %d\n",mio_major);
	  return(-EIO);
	}

	mio_reset();

	
	DBGout();
	return 0;

}

/***********************************************************************/
void cleanup_module(void)
{
  DBGin("cleanup");
  if (MOD_IN_USE)
    printk("MIO: device busy, remove delayed\n");
  else {
    /* nop */
    printk("MIO: remove module\n");
  }
   
  if (unregister_chrdev(mio_major, "atMIO16f") != 0) {
    printk("MIO: device busy or other module error \n");
  } else {
    printk("MIO: succesfully removed \n");
  } 
  DBGout();
}



#ifdef __cplusplus
}
#endif

