#include <mioP.h>

/***********************************************************************
 *
 *  Programmable Wave Generator
 *
 *
 * 
 *
 ***********************************************************************/




#if 0
struct wait_queue *WgenWait = NULL;
struct semaphore  WgenSem;
#endif

static int       WgenBaseFreq=CLK_10KHZ;
static int       WgenTimebase=0x000A;


volatile WaveTrace  Tracelist[MAX_TRACES];


volatile int              WgenStat     = 0;  /* generator state           */
volatile unsigned int     NumTraces    = 0;  /* Number of traces          */

volatile int              CurrentTrace = 0;  /* Current trace in process  */
volatile int              CurrentClick = 1;  /* curr. timer tick in trace */
volatile int              CurrentDac   = 0;  /* curr. dac value           */ 
volatile int              CurrentChan  = 0;  /* dac channel               */ 


/***********************************************************************/

int WavGen_Start(void){
DBGin("WavGen_Start");
    if( NumTraces <= 0) 
      return -ENOTRACES;

    if( WgenStat & WGEN_RUNNING || WgenStat & WGEN_PAUSED )  
    /* ouch! generator is running */
      return -EPROTO;

    /* set up driver state */
       CurrentTrace = 0;
       CurrentClick = 1;
       WgenStat |= WGEN_RUNNING;

    /* KICK HARDWARE */
    /* clear DAC circuit */
        outw( 0x0000, MIO_DUCLR);
        outw( 0x0000, MIO_TCINT);

    /* program counter 5 as interrupt generator */
        Am9513_SelectMode (  5, WgenBaseFreq );
        Am9513_SetTimebase(  5, WgenTimebase );
        Am9513_LoadArm(5);

    /* if enabled start simultaneous DAQ mode */
         if( DaqMode & DAQ_DACUP ){
	   outw(0x0000,MIO_ADCLR);
	   mio_DaqStart(DAQ_SCONV | DaqMode );
	 }

    /* program rtsi switch routing for trig signal using Counter 5 */
        mio_set_bitcmd( 0,(A4RCV | A4DRV | INTEN | DMAINTB1 | WGEN ));


DBGout();
}

/***************************************************/
int WavGen_Stop(void){
DBGin("WavGen_Stop");

    WgenStat &= ~WGEN_RUNNING;
    WgenStat &= ~WGEN_PAUSED;
    mio_reset_bitcmd( 0,(A4RCV | A4DRV | INTEN | DMAINTB1 | WGEN ));
    
    /* next interrupt will disable all */

DBGout();
}
/***********************************************************************/
int WavGen_Continue(void){
DBGin("WavGen_Continue");
    /* toggle run/pause mode */

    if( WgenStat & WGEN_PAUSED ){
      WgenStat |= WGEN_RUNNING;
      WgenStat &= ~WGEN_PAUSED;
    } 
    else {
      WgenStat &= ~WGEN_RUNNING;
      WgenStat |= WGEN_PAUSED;
    }

DBGout();
}



/*************************************************************/

int WavGen_IRQ(int status){

  if( WgenStat & WGEN_RUNNING ){
#if DEBUG
    if(dbgMask) printk("\nWGEN-INTERRUPT(%d)",CurrentDac);
#endif

    /* disable interupts */
    cli();

    /* Switch to next task (included to be faster) */

    if( !(WgenStat & WGEN_FORCE ) 
       && CurrentClick <= Tracelist[CurrentTrace].duration){
      /* switch to next tick */
      CurrentClick++;
    } else {
      /* switch next trace , if FORCE last task was JUMP */
      CurrentClick=1;
      if( !(WgenStat & WGEN_FORCE)) 
	CurrentTrace++; 
    }

    /* reset FORCE bit */
    WgenStat &= ~WGEN_FORCE;

    /* process next task */

    if( CurrentTrace < NumTraces ){                /* next task not empty */
     /* process current trace */    
      switch( Tracelist[CurrentTrace].type){
      case WAV_RAMP:
	CurrentDac+=Tracelist[CurrentTrace].arg;   /* add offset */
        outw(CurrentDac,MIO_DAC0 + CurrentChan );  /* set DAC */
	break;
      case WAV_STEP:
	CurrentDac=Tracelist[CurrentTrace].arg;
	outw(CurrentDac,MIO_DAC0 + CurrentChan );  /* set DAC */
	break;
      case WAV_RAW:
	CurrentDac=(Tracelist[CurrentTrace].buf)[CurrentClick -1];
	outw(CurrentDac,MIO_DAC0 + CurrentChan );  /* set DAC */
        break;
      case WAV_JMP:
	(Tracelist[CurrentTrace].duration)--; /*decrement jump counter */
        if( (Tracelist[CurrentTrace].duration) >= 0
	       && (Tracelist[CurrentTrace].arg < NumTraces) ){
	  CurrentTrace = Tracelist[CurrentTrace].arg;
	  WgenStat |= WGEN_FORCE;  
                                 /* force CurrentTrace to be next task */
	}
	break;
      default:
	printk("WGEN-INTERRUPT illegal trace type ?\n");
	break;
      }
      /* clear DACUP INT register */
      outw(0x0000, MIO_DUCLR );
      outw(0x0000, MIO_TCINT);

    } else {  

      /* STOP and CLEANUP the whole thing */

      /* disable interrupts and WGEN mode */
      mio_reset_bitcmd(0,(A4RCV | A4DRV | INTEN | DMAINTB1 | WGEN ));
      /* set status to stopped */
      WgenStat &= ~WGEN_RUNNING;
      /* clear DACUP INT register */
      outw(0x0000, MIO_DUCLR );
      outw(0x0000, MIO_TCINT );
#if 0
      if( WgenStat & WGEN_SYNC ){
	WgenStat &= ~WGEN_SYNC;
	wake_up_interruptible(&WgenWait);
      }
#endif
    }
  /* enable interrupts */
  sti();
  } else {
#if DEBUG
    if(dbgMask) printk("WGEN-INTERRUPT IDLE!\n");
#endif
    outw(0x0000, MIO_DUCLR );
    outw(0x0000, MIO_TCINT );
  }
}

/*************************************************************/
#if 0
int WavGen_WaitForCMPL(void){
DBGin("WavGen_Wait");
     if( WgenStat & WGEN_RUNNING ){
       /* enable interrupts */
       mio_set_bitcmd(0,(INTEN));
       /* semaphore set */
       /* sleep */
       WgenStat |= WGEN_SYNC;
       interruptible_sleep_on( &WgenWait );
     }
DBGout();
}
#endif

/*************************************************************/

/*************************************************************/

int WavGen_AddTrace(WaveTrace *trace ){

  int stat;

  WaveTrace *buf ;
  int       *rawbuf = NULL;    /* buffer for raw trace sequence */

  DBGin("WavGen_AddTrace");

  if( NumTraces >= MAX_TRACES ){
    printk(" Unable To add next Trace (exceeded) \n");
    return -ENOMEM;
  }

  if(( stat = verify_area(VERIFY_READ,trace,sizeof(WaveTrace)))!=0)
    return stat;

  if(( buf = (WaveTrace *) kmalloc( sizeof(WaveTrace), GFP_KERNEL)) == NULL) {
    printk("memory problem\n");
    return -ENOMEM;
  } 

  memcpy_fromfs(buf,trace,sizeof(WaveTrace));
  DBGprint(DBG_DATA,("AddTrace(type=%d,num=%d,dur=%d) adress 0x%lx",
		     buf->type,NumTraces,buf->duration,buf));     


  switch( buf->type ){
  case WAV_STEP:
    break;
  case WAV_RAMP:
    break;
  case WAV_JMP:
    break;
  case WAV_RAW:
    /* check if rawbuf readable */    
    if(( stat = verify_area(VERIFY_READ,buf->buf,buf->duration ))!=0){
      printk("Rawseq not readable");
      return stat;
    }
    if(( rawbuf = (int *) kmalloc( buf->duration + 1, GFP_KERNEL)) == NULL) {
      printk("memory problem alloc(rawseq)\n");
      return -ENOMEM;
    } 
    memcpy_fromfs(rawbuf , buf->buf , buf->duration );
    DBGprint(DBG_DATA,("Read Raw Trace @ 0x%x",rawbuf));
    break;

  default:
    printk(" Trying to Add illegal Trace Type \n");
    return -1;
  }

  Tracelist[NumTraces].type     = buf->type;
  Tracelist[NumTraces].duration = buf->duration;
  Tracelist[NumTraces].arg      = buf->arg;

  Tracelist[NumTraces].buf      = rawbuf;

  NumTraces++;


  kfree(buf);

  DBGout();return OK;
}

/*************************************************************/
int WavGen_Reset(void){
int i;
DBGin("WavGen_Reset");

if( NumTraces <= 0 ){
  printk("No Traces\n");
  return -ENOTRACES;
}

DBGprint(DBG_DATA,("NumTraces=%d",NumTraces));

if( WgenStat & WGEN_RUNNING ){
  WavGen_Stop();
}

for(i=0;i<NumTraces && i<MAX_TRACES && i>=0 ;i++){
  switch( Tracelist[i].type ){
  case WAV_RAW:
    if( Tracelist[i].buf != NULL ){
      DBGprint(DBG_DATA,("Free Raw Trace(type=%d,num=%d) adress 0x%lx",
			 Tracelist[i].type,i, Tracelist[i].buf ));     

      kfree( Tracelist[i].buf );
    }
    break;  
  case WAV_STEP:
  case WAV_RAMP:
  case WAV_JMP:
    DBGprint(DBG_DATA,("FreeTrace(type=%d,num=%d) adress 0x%lx",
		       Tracelist[i].type,i, &Tracelist[i]));     
    break;
  default:
    printk(" Trying to free illegal Trace Type \n");
    return -EWTRACE;
  }
}

NumTraces = 0;
CurrentChan = 0;
CurrentClick = 1;
CurrentTrace = 0;
WgenStat=0;


DBGprint(DBG_DATA,("NumTraces=%d",NumTraces));
DBGout();return OK;
}
/*************************************************************/

int WavGen_SetFreq(int freq){
DBGin("WavGen_SetFreq");
    switch( freq ){

    case CLK_5MHZ :
    case CLK_1MHZ :
    case CLK_100KHZ:
    case CLK_10KHZ:
    case CLK_1KHZ :
    case CLK_100HZ:
    case CLK_EXT :
      WgenBaseFreq=freq;
      break;
    default:
      return -EIMODE;
      break;
    }
return OK;
DBGout();
}

void WavGen_SetTimebase(int mask){
DBGin("WavGen_SetTimebase");
    WgenTimebase=mask;
DBGout();
}

WavGen_SetDac( int dac){
DBGin("WavGen_SetDac");
    CurrentDac=dac;
    outw(CurrentDac, MIO_DAC0 + CurrentChan );
DBGout();
}

WavGen_SetChan( int chan ){
DBGin("WavGen_SetChan");

CurrentChan = ( chan == 0 ? 0 : 2 );

DBGprint(DBG_DATA,("Choose Channel %d(adr+%d) for Wgen Mode",chan,CurrentChan));
DBGout();
}

/***********************************************************************/
int Wgen_Running(void){
return ((WgenStat & WGEN_RUNNING) ? 1 : 0 );
}
