#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>


#define ADC_REF 10.0

#include <tcl.h>
#include <mioP.h>

static int mio_fd = 0;
static float Gain = 1.0;

int Mio_Init ( Tcl_Interp *interp ){

extern int MioCmd _ANSI_ARGS_(( ClientData clientData,
			      Tcl_Interp *interp,
			       int argc,
			       char *argv[]
			       ));

    Tcl_CreateCommand(interp,"mio",MioCmd,
		         (ClientData) NULL,
			 (Tcl_CmdDeleteProc *) NULL );

    


    return TCL_OK;
}


/**********************************************************************/
int mio_open _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){


    if( mio_fd ){
      Tcl_AppendResult(interp,"MIO is open",(char *) NULL);
      return TCL_ERROR;
    }
    if(( mio_fd = open("/dev/atMIO",O_RDWR)) < 0){
      Tcl_AppendResult(interp,"Can't open MIO device!",(char *) NULL);
      return TCL_ERROR;
    }


    return TCL_OK;

}
/**********************************************************************/
int mio_close _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }
  
  close(mio_fd);
  mio_fd=0;
  return TCL_OK;

}


/**********************************************************************/
int mio_add _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

WaveTrace atrace;

  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }


  if( *argv[1]=='s' && !strcmp(argv[1],"sval")){
    if( argc < 3 ){
      Tcl_AppendResult(interp,"mio add sval ?value?",(char *) NULL);
      return TCL_ERROR;
    }

    ioctl(mio_fd,MIOWGENSDAC, atoi(argv[2]));
    return TCL_OK;
  }


  if( *argv[1]=='t' && !strcmp(argv[1],"trace")){
    if( argc < 3 ){
      Tcl_AppendResult(interp,"mio add trace ?type? ?args?",(char *) NULL);
      return TCL_ERROR;
    }
    if( *argv[2]=='r' && !strcmp(argv[2],"ramp")){
      if( argc < 5 ){
	Tcl_AppendResult(interp,"mio add trace ramp length offset",(char *) NULL);
	return TCL_ERROR;
      }
      atrace.type     = WAV_RAMP;
      atrace.duration = atoi(argv[3]);
      atrace.arg      = atoi(argv[4]);
      ioctl(mio_fd,MIOADDTRACE,(WaveTrace *) &atrace);
    }
    if( *argv[2]=='s' && !strcmp(argv[2],"step")){
      if( argc < 5 ){
	Tcl_AppendResult(interp,"mio add trace step length value",(char *) NULL);
	return TCL_ERROR;
      }
      atrace.type     = WAV_STEP;
      atrace.duration = atoi(argv[3]);
      atrace.arg      = atoi(argv[4]);
      ioctl(mio_fd,MIOADDTRACE,(WaveTrace *) &atrace);
    }

    if( *argv[2]=='j' && !strcmp(argv[2],"jump")){
      if( argc < 5 ){
	Tcl_AppendResult(interp,"mio add trace jump ?n? ?trace?",(char *) NULL);
	return TCL_ERROR;
      }
      atrace.type     = WAV_JMP;
      atrace.duration = atoi(argv[3]);
      atrace.arg      = atoi(argv[4]);
      ioctl(mio_fd,MIOADDTRACE,(WaveTrace *) &atrace);
    }




  }

  return TCL_OK;

}

/**********************************************************************/
int mio_get _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

AdcVal value;
char buf[80];


  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }

  if( *argv[1]=='r' && !strcmp(argv[1],"run")){
      if( ioctl(mio_fd,MIOWISRUN,0 )){
	Tcl_AppendResult(interp,"1",(char *) NULL);
      } else 
	Tcl_AppendResult(interp,"0",(char *) NULL);
  }
  if( *argv[1]=='a' && !strcmp(argv[1],"adc")){
      ioctl(mio_fd,MIOGETADC,(AdcVal *) &value );
      sprintf( buf, "%d", value.adc );
      Tcl_AppendResult(interp,buf,(char *) NULL);
  }
  if( *argv[1]=='w' && !strcmp(argv[1],"wdac")){
      ioctl(mio_fd,MIOGETADC,(AdcVal *) &value );
      sprintf( buf, "%d", value.dac0 );
      Tcl_AppendResult(interp,buf,(char *) NULL);
  }

  if( *argv[1]=='w' && !strcmp(argv[1],"wsample")){
      ioctl(mio_fd,MIOGETADC,(AdcVal *) &value );
      sprintf( buf, "%d %d", value.dac0 ,value.adc );
      Tcl_AppendResult(interp,buf,(char *) NULL);
  }
  
  return TCL_OK;

}

/**********************************************************************/
int mio_start _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }

  ioctl(mio_fd,MIOWGENSTART,0);

  return TCL_OK;
}
/*******************************************************************/
int mio_continue _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }

  ioctl(mio_fd,MIOWGENCONT,0);

  return TCL_OK;
}

/***********************************************************************/
int mio_stop _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }


    ioctl(mio_fd,MIOWGENSTOP,0);


  return TCL_OK;
}
/***********************************************************************/
int mio_reset _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }

  if( argc < 2 ){
      Tcl_AppendResult(interp," mio reset ?device? ",(char *) NULL);
      return TCL_ERROR;
  }

  if( *argv[1]=='w' && !strcmp(argv[1],"wgen")){
    ioctl(mio_fd,MIOWGENRESET,0);
  }

  return TCL_OK;
}

/***********************************************************************/
int mio_read _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

int *buf,i;
int count;
char dummy[50];
int retval;
char * NewValue;
char * NewValue1;
float v;

printf("read...in");
  if(! mio_fd ){
      Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
      return TCL_ERROR;
  }

  if( argc < 2 ){
      Tcl_AppendResult(interp," mio read ?device? ",(char *) NULL);
      return TCL_ERROR;
  }

  if( *argv[1]=='d' && !strcmp(argv[1],"daq")){
    if( argc < 5 ){
      Tcl_AppendResult(interp," mio read daq ?count? ?xlist? ?ylist?",(char *) NULL);
      return TCL_ERROR;
    }

    if(Tcl_GetInt(interp,argv[2],&count) != TCL_OK ){
      Tcl_AppendResult(interp," ?count? has to be integer ",(char *) NULL);
      return TCL_ERROR;
    }

    if(( buf = (int *) malloc( count * sizeof(int) +1 )) == NULL){
      Tcl_AppendResult(interp,"Memory Problem",(char *) NULL);
      return TCL_ERROR;
    }
printf("read...");
    /* read daq buffer */
    if(( count = read(mio_fd,buf,count)) < 0){
      free(buf);
      return TCL_ERROR;
    }
printf("count=%d\n",count);
    for( i = 0; i< count; i++ ){              /* read X values (CurrentDac) */
      sprintf(dummy,"%d.0",(short) (buf[i]>>16) );
      /*printf("buf>>16 = %d",(short) (buf[i]>>16));*/
      NewValue1 = Tcl_SetVar(interp, argv[3],dummy, 
			   TCL_LIST_ELEMENT | TCL_APPEND_VALUE 
			   | TCL_LEAVE_ERR_MSG);
      if ( NewValue1 == NULL ){
	free(buf);
	return TCL_ERROR;
      }
    }
printf("gain=%f",Gain);
    for( i = 0; i< count; i++ ){            /* read Y values */
      v = (((short) buf[i] ) / (float) 2048 ) *  10.0 / Gain ;
      sprintf(dummy,"%f",v );
      NewValue = Tcl_SetVar(interp, argv[4],dummy, 
			   TCL_LIST_ELEMENT | TCL_APPEND_VALUE 
			   | TCL_LEAVE_ERR_MSG);
      if ( NewValue == NULL ){
	free(buf);
	return TCL_ERROR;
      }
    }

    free(buf);
    interp->result = NewValue;
    return TCL_OK;
  } 
  Tcl_AppendResult(interp,"mio read ?device? ?options? ",(char *) NULL);
  return TCL_ERROR;
}





/**********************************************************************/
int mio_set _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp, int argc,char *argv[])){

  int mux_chan=0;
  int mux_gain=0;
  int i,mode;

  if(! mio_fd ){
    Tcl_AppendResult(interp,"MIO not open",(char *) NULL);
    return TCL_ERROR;
  }

  if( argc < 3 ){
    Tcl_AppendResult(interp,"mio set ?device? value",(char *) NULL);
    return TCL_ERROR;
  }

  /* DAC Channels */

  if( *argv[1]=='d' && !strcmp(argv[1],"dac0")){
    ioctl(mio_fd,MIOSETDAC0, atoi(argv[2]));
  }
  if( *argv[1]=='d' && !strcmp(argv[1],"dac1")){
    ioctl(mio_fd,MIOSETDAC1, atoi(argv[2]));
  }

  /* Debugging mode */
  if( *argv[1]=='d' && !strcmp(argv[1],"debug")){
    ioctl(mio_fd,MIODEBUG, atoi(argv[2]));
  }


  /* DAQ modes */
  if( *argv[1]=='d' && !strcmp(argv[1],"daq")){
  
    if( argc < 4 ){
      Tcl_AppendResult(interp,"mio set daq ?mode? ?args?",(char *) NULL);
      return TCL_ERROR;
    }

    if( *argv[2]=='m' && !strcmp(argv[2],"mode")){
      if( argc < 4 ){
	Tcl_AppendResult(interp,"mio set daq mode ?modes?",(char *) NULL);
	return TCL_ERROR;
      }
      
      mode = 0;
      for(i=3;i<argc;i++){
	printf(" %s | ",argv[i]);
        if( *argv[i]=='d' && !strcmp(argv[i],"dacup"))
	  mode |= DAQ_DACUP;
	else  if( *argv[i]=='s' && !strcmp(argv[i],"sconv"))
	  mode |= DAQ_SCONV;
	else  if( *argv[i]=='b' && !strcmp(argv[i],"bipolar"))
	  mode |= DAQ_BIPOLAR;
	else  if( *argv[i]=='d' && !strcmp(argv[i],"diff"))
	  mode |= DAQ_DIFF;
      }
printf("mode=0x%x",mode);
      ioctl(mio_fd,MIOSETMODE, mode );
    }


   return TCL_OK;
  }




  /* Wavegenerator modes */

  if( *argv[1]=='w' && !strcmp(argv[1],"wgen")){
  
    if( argc < 4 ){
      Tcl_AppendResult(interp,"mio set wgen ?mode? ?arg?",(char *) NULL);
      return TCL_ERROR;
    }

    if( *argv[2]=='c' && !strcmp(argv[2],"clock")){
      if( !strcmp(argv[3],"5mhz"))
	ioctl(mio_fd,MIOWGENSFREQ, CLK_5MHZ );
      else if( !strcmp(argv[3],"1mhz"))
	ioctl(mio_fd,MIOWGENSFREQ, CLK_1MHZ );
      else if( !strcmp(argv[3],"100khz"))
	ioctl(mio_fd,MIOWGENSFREQ, CLK_100KHZ );
      else if( !strcmp(argv[3],"10khz"))
	ioctl(mio_fd,MIOWGENSFREQ, CLK_10KHZ );
      else if( !strcmp(argv[3],"1khz"))
	ioctl(mio_fd,MIOWGENSFREQ, CLK_1KHZ );
      else if( !strcmp(argv[3],"100hz"))
	ioctl(mio_fd,MIOWGENSFREQ, CLK_100HZ );
      else if( !strcmp(argv[3],"external"))
	ioctl(mio_fd,MIOWGENSFREQ, CLK_EXT );
      else {
	Tcl_AppendResult(interp,"illegal clock mode, must be one of:\n 1mhz,100khz,10khz,1khz,100hz,external ",(char *) NULL);
	return TCL_ERROR;
      }
    }

    if( *argv[2]=='b' && !strcmp(argv[2],"base")){
	ioctl(mio_fd,MIOWGENSTIM, atoi(argv[3]) );
    }
	    
    if( *argv[2]=='c' && !strcmp(argv[2],"chan")){
	ioctl(mio_fd,MIOWGENSCHA, atoi(argv[3]) );
    }


  }	      

  /* MUX Circuit */

  if( *argv[1]=='m' && !strcmp(argv[1],"mux")){

    if( argc < 4 ){
      Tcl_AppendResult(interp,"mio set mux ?channel? ?gain?",(char *) NULL);
      return TCL_ERROR;
    }
    mux_chan = atoi(argv[2]);
    mux_gain = atoi(argv[3]);
      switch (mux_gain) {
      case 0:
	Gain = 0.5;
	break;
      case 1:
	Gain = 1.0;
	break;
      case 2:
	Gain = 2.0;
	break;
      case 3:
	Gain = 5.0;
	break;
      case 4:
	Gain = 10.0;
	break;
      case 5:
	Gain = 20.0;
	break;
      case 6:
	Gain = 50.0;
	break;
      case 7:
	Gain = 100.0;
	break;
      default: 
	 return TCL_ERROR;

      }

    ioctl(mio_fd,MIOSETMUX, ((mux_gain<<1) & ~0x01) | ((mux_chan & 0xf )<<4)       );
  }
  return TCL_OK;

}
/**********************************************************************/

/**********************************************************************/

int MioCmd _ANSI_ARGS_(( ClientData clientData,
			Tcl_Interp *interp,
			int argc,
			char *argv[]
			)) 
{
  if( *argv[1]=='s' && !strcmp(argv[1],"stop")){
    return mio_stop ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='p' && !strcmp(argv[1],"pause")){
    return mio_continue ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='s' && !strcmp(argv[1],"start")){
    return mio_start ( clientData, interp, argc-1,argv+1 );
  }


  else if( *argv[1]=='o' && !strcmp(argv[1],"open")){
    return mio_open  ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='c' && !strcmp(argv[1],"close")){
    return mio_close ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='s' && !strcmp(argv[1],"set")){
    return mio_set ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='g' && !strcmp(argv[1],"get")){
    return mio_get ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='r' && !strcmp(argv[1],"reset")){
    return mio_reset ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='a' && !strcmp(argv[1],"add")){
    return mio_add ( clientData, interp, argc-1,argv+1 );
  }
  else if( *argv[1]=='r' && !strcmp(argv[1],"read")){
    return mio_read ( clientData, interp, argc-1,argv+1 );
  }
  


  Tcl_AppendResult(interp,"mio ?cmd? ?args? ",(char *)NULL);
  return TCL_ERROR;
}


























